@@ -6,14 +6,13 @@ import atexit
 import logging
 import os
 import sys
-import time
 import threading
 import ctypes
-from ctypes import cdll, CDLL, PyDLL, CFUNCTYPE, c_char_p, c_void_p, string_at
+from ctypes import cdll, CDLL, CFUNCTYPE, c_char_p, c_void_p, string_at
 from ctypes.util import find_library
 
 import ftools
-from ctypes_declarations import prototype
+from ctypes_declarations import prototype, EXCEPTION_HANDLER_FUNCTYPE
 
 
 
@@ -44,7 +43,7 @@ def load_dll(libname, fallbacks=None):
             "Could not find library %s or load any of its variants %s" % (
                 libname, fallbacks or []))
        
-if sys.platform == 'linux2':
+if sys.platform.startswith('linux'):
     _lgeos = load_dll('geos_c', fallbacks=['libgeos_c.so.1', 'libgeos_c.so'])
     free = load_dll('c').free
     free.argtypes = [c_void_p]
@@ -93,8 +92,12 @@ else: # other *nix systems
 
 def _geos_c_version():
     func = _lgeos.GEOSversion
+    func.argtypes = []
     func.restype = c_char_p
     v = func().split('-')[2]
+    # Ditch any SVN revision numbering that may have crept in without (ftm)
+    # importing `re`.
+    v = v.split()[0]
     return tuple(int(n) for n in v.split('.'))
 
 geos_capi_version = geos_c_version = _geos_c_version()
@@ -130,7 +133,7 @@ if geos_c_version >= (1,5,0):
     
     # Handle special case.
     _lgeos.initGEOS_r.restype = c_void_p
-    _lgeos.initGEOS_r.argtypes = [c_void_p, c_void_p]
+    _lgeos.initGEOS_r.argtypes = [EXCEPTION_HANDLER_FUNCTYPE, EXCEPTION_HANDLER_FUNCTYPE]
     _lgeos.finishGEOS_r.argtypes = [c_void_p]
 
 # Exceptions
@@ -149,11 +152,11 @@ class PredicateError(Exception):
 
 def error_handler(fmt, list):
     LOG.error("%s", list)
-error_h = CFUNCTYPE(None, c_char_p, c_char_p)(error_handler)
+error_h = EXCEPTION_HANDLER_FUNCTYPE(error_handler)
 
 def notice_handler(fmt, list):
     LOG.warning("%s", list)
-notice_h = CFUNCTYPE(None, c_char_p, c_char_p)(notice_handler)
+notice_h = EXCEPTION_HANDLER_FUNCTYPE(notice_handler)
 
 def cleanup():
     if _lgeos is not None :
@@ -167,7 +170,7 @@ def errcheck_wkb(result, func, argtuple):
     if not result:
         return None
     size_ref = argtuple[-1]
-    size = size_ref._obj
+    size = size_ref.contents
     retval = ctypes.string_at(result, size.value)[:]
     lgeos.GEOSFree(result)
     return retval
